from yaml import load


class Route(object):
    controller_values = [
        'template',
        'permission',
        'attr',
        'renderer',
        'http_cache',
        'wrapper',
        'decorator',
        'mapper',
        'context',
        'request_type',
        'request_method',
        'request_param',
        'match_param',
        'containment',
        'xhr',
        'accept',
        'header',
        'path_info',
        'check_csrf',
        'physical_path',
        'effective_principals',
        'custom_predicates',
        'predicates',
    ]

    def __init__(self, app, prefix=None):
        self.app = app
        self.prefix = prefix
        self.app.config.registry['route'] = self
        self.routes = {}

    @property
    def config(self):
        return self.app.config

    def add(self, controller, route, url, *args, **kwargs):
        self.config.add_route(
            route,
            url,
            *args,
            **kwargs)

        self.add_view(controller, route_name=route)

    def add_view(self, controller, route=None, **kwargs):
        url = self.convert_url(controller)

        controller_class = self.config.maybe_dotted(url)

        if 'route_name' in kwargs:
            self.routes[kwargs['route_name']] = controller_class

        for name in self.controller_values:
            self.set_controller_config(kwargs, controller_class, name)

        self.config.add_view(url, **kwargs)

    def convert_url(self, url):
        return self.prefix + url

    def set_controller_config(self, kwargs, controller, name):
        value = getattr(controller, name, None)
        if name == 'template' and value is not None:
            name, value = self._convert_template(name, value, controller)
        if value:
            kwargs[name] = value

    def _convert_template(self, name, value, controller):
        templates_dir = getattr(controller, 'templates_dir', 'templates')
        app, path = value.split(':')
        value = "%s.%s:%s/%s" % (
            self.app.module,
            app,
            templates_dir,
            path)
        return 'renderer', value

    def read_yaml(self, path):
        with open(path, 'r') as stream:
            data = load(stream)
        for name, value in self._inner_dict(data):
            value['controller'] = '%s.%s' % (
                name,
                value['controller'],)
            self.add(**value)

    def _inner_dict(self, data, prefix=''):
        for name, value in data.items():
            if type(value) is dict:
                yield from self._inner_dict(value, prefix + name + '.')
            else:
                for element in value:
                    yield (prefix + name, element)



from os import path

from baelfire.template import TemplateTask


class GeneratedOnceTemplateTask(TemplateTask):

    def __init__(self, *args, **kwargs):
        kwargs['check_template'] = False
        super().__init__(*args, **kwargs)


class InitPy(GeneratedOnceTemplateTask):
    path = '/templates/initpy'

    def get_template_path(self):
        return path.join('project/init.py.tpl')

    def get_output_file(self):
        return self.paths['project:initpy']


class ManagePy(GeneratedOnceTemplateTask):
    path = '/templates/manage'

    def get_template_path(self):
        return path.join('project/manage.py.tpl')

    def get_output_file(self):
        return self.paths['project:managepy']


class Routes(GeneratedOnceTemplateTask):
    path = '/templates/routes'

    def get_template_path(self):
        return path.join('project/routes.py.tpl')

    def get_output_file(self):
        return self.paths['project:routes']


class Settings(GeneratedOnceTemplateTask):
    path = '/templates/settings'

    def get_template_path(self):
        return path.join('project/settings.py.tpl')

    def get_output_file(self):
        return self.paths['project:default']


class FrontendIni(TemplateTask):
    name = 'Creating frontend.ini file'
    path = '/frontend.ini'

    def generate_links(self):
        super().generate_links()
        self.add_link('bael.hatak.tasks:CreateDataDir')
        self.add_link('bael.hatak.tasks:BaelfireInitFile')
        self.add_link('bael.project.tasks:GatherData')

    def get_output_file(self):
        return self.paths['data:frontend.ini']

    def get_template_path(self):
        return 'frontend.ini.tpl'


class TestFixtures(GeneratedOnceTemplateTask):
    path = '/templates/testfixtures'

    def get_template_path(self):
        return path.join('project/fixtures.py.tpl')

    def get_output_file(self):
        return self.paths['application:fixtures']


class TestCases(GeneratedOnceTemplateTask):
    path = '/templates/testcases'

    def get_template_path(self):
        return path.join('project/cases.py.tpl')

    def get_output_file(self):
        return self.paths['application:cases']


class TestSettings(GeneratedOnceTemplateTask):
    path = '/templates/testsettings'

    def get_template_path(self):
        return path.join('project/testsettings.py.tpl')

    def get_output_file(self):
        return self.paths['project:testsettings']


class RedmeFile(GeneratedOnceTemplateTask):
    path = '/templates/readmefile'

    def get_template_path(self):
        return path.join('project/readme.txt.tpl')

    def get_output_file(self):
        return self.paths['readmefile']



class UnpackRequest(object):

    def __init__(self):
        self.unpackers = {}

    def add(self, name, method):
        self.unpackers[name] = method

    def __call__(self, obj, request):
        for key, unpacker in self.unpackers.items():
            value = unpacker(request)
            setattr(obj, key, value)


def unpack(obj, request):
    obj.request = request
    request.registry['unpacker'](obj, request)



from pyramid.config import Configurator
from smallsettings import Factory

from .unpackrequest import UnpackRequest
from .command import CommandsApplication
from .errors import PluginNotFound


class Application(object):

    """Prepering and configuring project."""

    def __init__(self, module, make_routes):
        self.make_routes = make_routes
        self.module = module
        self.initialize_unpacker()
        self.plugins = []
        self.plugin_types = []
        self.controller_plugins = []
        self.commands = None

    def initialize_unpacker(self):
        self.unpacker = UnpackRequest()
        self.unpacker.add('POST', lambda req: req.POST)
        self.unpacker.add('GET', lambda req: req.GET)
        self.unpacker.add('matchdict', lambda req: req.matchdict)
        self.unpacker.add('settings', lambda req: req.registry['settings'])
        self.unpacker.add('registry', lambda req: req.registry)
        self.unpacker.add('route', lambda req: req.route_path)

    def add_plugin(self, plugin):
        if type(plugin) in self.plugin_types:
            # add plugin only once
            # TODO: this should raise an error
            return
        plugin.init(self)
        plugin.add_unpackers()
        plugin.add_controller_plugins()
        self.plugin_types.append(type(plugin))
        self.plugins.append(plugin)

    def __call__(self, settings={}):
        self.settings = self.generate_settings(settings)
        self.make_before_config()
        self.create_config()
        self.make_after_config()
        self.make_pyramid_includes()
        self.make_routes(self)
        self.make_registry(self.config.registry)

        return self.config.make_wsgi_app()

    def run_commands(self, settings={}):
        self.settings = self.generate_settings(settings)
        self.commands = CommandsApplication(self)
        self.commands()

    def generate_settings(self, settings):
        self._settings, self._paths = self.get_settings(self.module, settings)
        merged = self._settings.merged(self._paths)
        return merged.to_dict()

    @classmethod
    def get_settings(cls, module, settings={}, additional_modules=None):
        additional_modules = additional_modules or [
            ('local', False),
        ]
        factory = Factory('%s.application' % (module))
        settings, paths = factory.make_settings(
            settings=settings,
            additional_modules=additional_modules,)
        return settings, paths

    @classmethod
    def get_settings_for_tests(cls, module, settings={}):
        additional_modules = [
            ('local', False),
            ('tests', True),
            ('local_test', False),
        ]

        return cls.get_settings(module, settings, additional_modules)

    def create_config(self):
        self.config = Configurator(
            settings=self.settings,
        )

    def make_before_config(self):
        for plugin in self.plugins:
            plugin.before_config()

    def make_after_config(self):
        for plugin in self.plugins:
            plugin.add_request_plugins()
        for plugin in self.plugins:
            plugin.after_config()

    def make_pyramid_includes(self):
        for plugin in self.plugins:
            plugin.make_config_include_if_able()
        self.config.commit()

    def make_registry(self, registry):
        registry['unpacker'] = self.unpacker
        registry['settings'] = self.settings
        registry['controller_plugins'] = self.controller_plugins
        for plugin in self.plugins:
            plugin.add_to_registry()

    def add_controller_plugin(self, plugin):
        self.controller_plugins.append(plugin)

    def _validate_dependency_plugin(self, plugin):
        if not plugin in self.plugin_types:
            raise PluginNotFound(plugin)



from glob import glob
from os import path

from bael.project.virtualenv import VirtualenvTask
from baelfire.dependencies import (
    AlwaysRebuild,
    FileChanged,
)


class AlembicBase(VirtualenvTask):

    def alembic(self, command, *args, **kwargs):
        command = self.paths['exe:manage'] + ' alembic ' + command
        return self.python(command, *args, **kwargs)


class AlembicData(AlembicBase):
    name = 'Creating alembic directory'
    path = '/alembic/data'

    def generate_links(self):
        self.add_link('bael.hatak.tasks:ProjectTemplates')

    def get_output_file(self):
        return self.paths['alembic:main']

    def make(self):
        self.alembic('init')


class AlembicMigration(AlembicBase):
    name = 'Apply migration'
    path = '/alembic/migration'

    def generate_dependencies(self):
        super().generate_dependencies()
        for file_ in glob(path.join(self.paths['alembic:versions'], '*.py')):
            self.add_dependecy(FileChanged(file_))

    def get_output_file(self):
        return self.paths['flags:dbmigration']

    def generate_links(self):
        self.add_link(AlembicData)

    def make(self):
        self.alembic('upgrade %s' % (self.kwargs.get('revision', 'head')))
        self.touchme()


class AlembicRevision(AlembicBase):
    name = 'Create new revision script'
    path = '/alembic/script'

    def generate_dependencies(self):
        self.add_dependecy(AlwaysRebuild())

    def generate_links(self):
        self.add_link(AlembicData)

    def make(self):
        description = self.ask_for('description', 'Migration description')
        self.alembic('revision -m "%s"' % (description))



class Plugin(object):

    @property
    def config(self):
        return self.app.config

    @property
    def registry(self):
        return self.config.registry

    @property
    def settings(self):
        return self.app.settings

    @property
    def unpacker(self):
        return self.app.unpacker

    @property
    def controller(self):
        return self.app.controller_plugins

    def init(self, app):
        self.app = app
        self.validate_plugin()

    def make_config_include_if_able(self):
        try:
            self.config.include(self.get_include_name())
        except NotImplementedError:
            pass

    def get_include_name(self):
        raise NotImplementedError()

    def add_to_registry(self):
        pass

    def before_config(self):
        pass

    def after_config(self):
        pass

    def add_unpackers(self):
        pass

    def add_controller_plugins(self):
        pass

    def add_controller_plugin(self, plugin):
        def append_parent_wrapper(controller):
            return plugin(self, controller)
        self.controller.append(append_parent_wrapper)

    def add_commands(self, parent):
        pass

    def validate_plugin(self):
        pass

    def add_request_plugins(self):
        pass

    def add_request_plugin(self, plugin):
        plugin = plugin()
        plugin.set_parent(self)
        self.config.add_request_method(
            plugin.init,
            plugin.name,
            reify=True)


class RequestPlugin(object):

    def __init__(self, name):
        self.name = name
        self._block = False

    def set_parent(self, parent):
        self.parent = parent

    def init(self, request):
        self.request = request
        self.POST = request.POST
        self.GET = request.GET
        self.matchdict = request.matchdict
        self.settings = request.registry['settings']
        self.registry = request.registry
        self.route = request.route_path
        return self.return_once()

    def return_once(self):
        return self


def reify(method):
    """Decorator for making reify methods with request instance for request."""

    def requester(self, request):
        def on_request(*args, **kwargs):
            return method(self, request, *args, **kwargs)
        return on_request
    return requester



from os import mkdir, path

from bael.project.virtualenv import VirtualenvTask
from baelfire.application.commands.init.models import InitFile
from baelfire.dependencies import (
    AlwaysRebuild,
    FileDoesNotExists,
)
from baelfire.task import Task


class CreateDataDir(Task):
    name = 'Creating data directory'
    path = '/data'

    directorie_names = [
        'data',
    ]

    @property
    def directories(self):
        for name in self.directorie_names:
            yield self.paths[name]

    def generate_dependencies(self):
        for directory in self.directories:
            self.add_dependecy(FileDoesNotExists(directory))

    def make(self):
        for directory in self.directories:
            if not path.exists(directory):
                mkdir(directory)


class Develop(Task):

    def generate_links(self):
        self.add_link('bael.hatak.templates:FrontendIni')
        self.add_link('bael.project.virtualenv:Develop')
        self.add_link('bael.hatak.alembic:AlembicMigration')


class CommandTask(VirtualenvTask):

    def generate_dependencies(self):
        self.add_dependecy(AlwaysRebuild())

    def generate_links(self):
        self.add_link(Develop)


class Serve(CommandTask):
    name = 'Run development server'
    path = '/serve'

    def make(self):
        self.pserve('%(data:frontend.ini)s --reload' % (self.paths))

    def pserve(self, command, *args, **kwargs):
        command = self.paths['exe:pserve'] + ' ' + command
        return self.command([command], *args, **kwargs)


class Shell(VirtualenvTask):
    name = 'Run development shell'
    path = '/shell'

    def generate_dependencies(self):
        self.add_dependecy(AlwaysRebuild())

    def generate_links(self):
        self.add_link(Develop)

    def make(self):
        self.pshell('%(data:frontend.ini)s' % (self.paths))

    def pshell(self, command, *args, **kwargs):
        command = self.paths['exe:pshell'] + ' ' + command
        return self.command([command], *args, **kwargs)


class BaelfireInitFile(Task):

    path = '/initfile'

    def get_output_file(self):
        return InitFile.filename

    def make(self):
        initfile = InitFile()
        initfile.assign('bael.hatak.recipe:HatakRecipe')
        initfile.save()


class ProjectTemplates(Task):

    path = '/templates'

    def generate_links(self):
        self.add_link('bael.hatak.templates:InitPy')
        self.add_link('bael.hatak.templates:ManagePy')
        self.add_link('bael.hatak.templates:Routes')
        self.add_link('bael.hatak.templates:Settings')
        self.add_link('bael.hatak.templates:TestFixtures')
        self.add_link('bael.hatak.templates:TestCases')
        self.add_link('bael.hatak.templates:TestSettings')
        self.add_link('bael.hatak.templates:RedmeFile')


class Tests(CommandTask):
    name = 'Run tests'
    path = '/tests'

    def make(self):
        if 'g' in self.kwargs:
            self.tests('-g %s' % (self.kwargs['g'][0]))
        elif 'c' in self.kwargs:
            self.tests('-c %s' % (self.kwargs['c'][0]))
        else:
            self.tests()

    def tests(self, command='', *args, **kwargs):
        command = self.paths['exe:manage'] + ' tests ' + command
        return self.command([command], *args, **kwargs)


class TestsList(Tests):
    name = 'Show all tests'
    path = '/tests/list'

    def make(self):
        self.tests('-l')


class Coverage(CommandTask):
    name = 'Generate test coverage report'
    path = '/tests/coverage'

    def make(self):
        omits = ','.join(self.settings['coverage omits'])
        self.coverage('run --branch %(exe:manage)s tests' % self.paths, True)
        self.coverage('html --omit=%s' % (omits,), True)
        if 'browser' in self.kwargs:
            browser = self.kwargs['browser'][0]
            self.command(['%s htmlcov/index.html' % (browser)])

    def coverage(self, command='', *args, **kwargs):
        command = self.paths['exe:coverage'] + ' ' + command
        return self.command([command], *args, **kwargs)



import sys


class CommandsApplication(object):

    def __init__(self, app):
        self.app = app
        self.commands = {}
        self.namespaces = {}

        for plugin in self.app.plugins:
            plugin.add_commands(self)

    def add_command(self, command):
        command.init(self)

        namespace = self.namespaces.get(command.namespace, {})
        namespace[command.name] = command
        self.namespaces[command.namespace] = namespace
        self.commands[command.name] = command

    def __call__(self):
        if len(sys.argv) == 1:
            self.print_list()
        else:
            self.run_command()

    def print_list(self):
        print("All commands:")
        for namespace, commands in self.namespaces.items():
            print('[%s]' % (namespace,))
            for name, command in commands.items():
                print('    %s\t%s' % (name, command.help or ''))

    def run_command(self):
        name = sys.argv[1]
        self.commands[name](sys.argv[1:])


class Command(object):

    def __init__(self, namespace, name, help=None):
        self.namespace = namespace
        self.name = name
        self.help = help

    def init(self, parent):
        self.parent = parent
        self.app = parent.app



from baelfire.task import Task
from baelfire.dependencies import AlwaysRebuild, PidFileIsRunning
from baelfire.dependencies import PidFileIsNotRunning


class UwsgiBase(Task):

    def uwsgi(self, command, *args, **kwargs):
        command = self.paths['exe:uwsgi'] + ' ' + command
        return self.command([command], *args, **kwargs)


class UwsgiStart(UwsgiBase):
    path = '/uwsgi/start'

    def generate_links(self):
        self.add_link('bael.hatak.tasks:Develop')

    def generate_dependencies(self):
        self.add_link('bael.hatak.tasks:Develop')
        self.add_dependecy(PidFileIsNotRunning(self.paths['uwsgi:pid']))

    def make(self):
        self.uwsgi('--ini-paste %(data:frontend.ini)s' % (self.paths))


class UwsgiStop(UwsgiBase):
    path = '/uwsgi/stop'

    def generate_dependencies(self):
        self.add_dependecy(PidFileIsRunning(self.paths['uwsgi:pid']))

    def make(self):
        with open(self.paths['uwsgi:pid'], 'r') as pidfile:
            pid = pidfile.read()
        self.command('kill -INT %s' % (pid,))


class UwsgiRestart(UwsgiStart):
    path = '/uwsgi/restart'

    def generate_dependencies(self):
        self.add_dependecy(AlwaysRebuild())

    def generate_links(self):
        super().generate_links()
        self.add_link(UwsgiStop)



from pyramid.httpexceptions import HTTPFound
from .unpackrequest import unpack


class EndController(Exception):
    pass


class Controller(object):

    def __init__(self, root_factory, request):
        self.request = request
        self.root_factory = root_factory
        unpack(self, request)

        self.response = None
        self.initialize_plugins()

    def __call__(self):
        try:
            self.before_filter()
            self.data = self.generate_default_data()
            data = self.make() or {}
            self.data.update(data)
            self.after_filter()
            if self.response is None:
                self.make_helpers()
                self.make_plugin_helpers()
                return self.data
            else:
                return self.response
        except EndController:
            return self.response

    def generate_default_data(self):
        data = {
            'request': self.request,
            'route': self.request.route_path,
        }
        for plugin in self.plugins:
            plugin.generate_default_data(data)
        return data

    def make(self):
        pass

    def initialize_plugins(self):
        self.plugins = []
        for plugin in self.registry['controller_plugins']:
            self.plugins.append(plugin(self))

    def before_filter(self):
        for plugin in self.plugins:
            plugin.before_filter()

    def after_filter(self):
        for plugin in self.plugins:
            plugin.after_filter()

    def redirect(self, to):
        url = self.request.route_url(to)
        self.response = HTTPFound(location=url)

    def add_helper(self, name, cls, *args, **kwargs):
        self.data[name] = cls(self.request, *args, **kwargs)

    def make_helpers(self):
        pass

    def make_plugin_helpers(self):
        for plugin in self.plugins:
            plugin.make_helpers()


class ControllerPlugin(object):

    def __init__(self, parent, controller):
        self.parent = parent
        self.controller = controller
        self.request = self.controller.request
        unpack(self, self.request)
        self.add_helper = self.controller.add_helper
        self.add_controller_methods()

    def add_controller_methods(self):
        pass

    def add_method(self, name):
        method = getattr(self, name)
        setattr(self.controller, name, method)

    def before_filter(self):
        pass

    def after_filter(self):
        pass

    def generate_default_data(self, data):
        pass

    def make_helpers(self):
        pass



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
    'bael.project==0.1.2',
    'pyramid',
    'pyyaml',
]


if __name__ == '__main__':
    setup(
        name='hatak',
        version='0.2.1',
        description='Small Pyramid extension/project managment',
        license='Apache License 2.0',
        packages=find_packages('src'),
        package_dir={'': 'src'},
        install_requires=install_requires,
        namespace_packages=['bael'],
        zip_safe=False,
        package_data={
            '': [
                'hatak/templates/*.ini',
                'hatak/templates/*.tpl',
                'hatak/templates/project/*.tpl'],
        },
        entry_points="""\
          [console_scripts]
              hatak = bael.hatak.recipe:run
        """,
        )



from bael.project.recipe import ProjectRecipe
from baelfire.application.application import Application
from baelfire.recipe import Recipe

from .tasks import (
    CreateDataDir,
    Serve,
    BaelfireInitFile,
    ProjectTemplates,
    Develop,
    Shell,
    Tests,
    TestsList,
    Coverage,
)

from .templates import (
    InitPy,
    Routes,
    Settings,
    FrontendIni,
    TestFixtures,
    TestCases,
    TestSettings,
    RedmeFile,
    ManagePy,
)

from .uwsgi import (
    UwsgiStart,
    UwsgiStop,
    UwsgiRestart,
)

from .alembic import (
    AlembicData,
    AlembicMigration,
    AlembicRevision,
)


class HatakRecipe(Recipe):

    prefix = '/hatak'

    def create_settings(self):
        self.set_path('project:src', 'cwd', 'src')
        self.set_path('data', 'cwd', 'data')
        self.set_path('data:frontend.ini', 'data', 'frontend.ini')
        self.set_path('data:log', 'data', 'all.log')
        self.set_path('uwsgi:socket', None, '/tmp/uwsgi.socket')
        self.set_path('uwsgi:pid', 'data', 'uwsgi.pid')
        self.set_path('uwsgi:log', 'data', 'uwsgi.log')
        self.set_path(
            'venv:site-packages',
            'virtualenv_path',
            'lib/python3.4/site-packages/')

        self.set_path('flags:dbversioning', 'flags', 'versioning.flag')
        self.set_path('flags:dbmigration', 'flags', 'dbmigration.flag')

        self.set_path('migration:main', 'cwd', 'migrations')
        self.set_path('migration:manage', 'migration:main', 'manage2.py')
        self.set_path('migration:versions', 'migration:main', 'versions')

        self.set_path('exe:pserve', 'virtualenv:bin', 'pserve')
        self.set_path('exe:pshell', 'virtualenv:bin', 'pshell')
        self.set_path('exe:uwsgi', 'virtualenv:bin', 'uwsgi')
        self.set_path(
            'exe:manage', 'virtualenv:bin', '%(package:name)s_manage')
        self.set_path('exe:coverage', 'virtualenv:bin', 'coverage')

        self.settings['develop'] = True

        self.set_path('project:application', 'project:home', 'application')
        self.set_path('project:initpy', 'project:application', 'init.py')
        self.set_path('project:managepy', 'project:application', 'manage.py')
        self.set_path('project:settings', 'project:application', 'settings')
        self.set_path('readmefile', 'cwd', 'README.txt')
        self.set_path('application:tests', 'project:application', 'tests')
        self.set_path(
            'application:fixtures', 'application:tests', 'fixtures.py')
        self.set_path(
            'application:cases', 'application:tests', 'cases.yml')

        self.set_path('project:routes', 'project:application', 'routes.py')
        self.set_path('project:default', 'project:settings', 'default.py')
        self.set_path('project:testsettings', 'project:settings', 'tests.py')

        self.set_path('alembic:ini', 'data', 'alembic.ini')
        self.set_path('alembic:main', 'cwd', 'alembic')
        self.set_path('alembic:versions', 'alembic:main', 'versions')

        self.settings['coverage omits'] = [
            'eggs/*',
            '*/venv/*',
            '*/tests/*',
            '*/migrations/*',
            '*/routes.py',
            '*/settings/*',
        ]

    def final_settings(self):
        self.set_path('virtualenv_path', 'cwd', 'venv')
        self.set_path('flags', 'data', 'flags')

        self.settings['packages'] = [
            'hatak==0.2',
            'coverage',
            'hatak_logging',
            'hatak_jinja2',
            'hatak_haml',
            'hatak_sql',
            'hatak_alembic',
            'hatak_beaker',
            'hatak_debugtoolbar',
            'hatak_toster',
            'hatak_statics',

            'waitress',
            'uwsgi',
        ]
        self.settings['directories'].append('project:application')
        self.settings['directories'].append('project:settings')
        self.settings['directories'].append('application:tests')
        self.settings['entry_points'] = (
            '\r\t[paste.app_factory]\n'
            '\t\tmain = %(package:name)s.application.init:main\n'
            '\t[console_scripts]\n'
            '\t\t%(package:name)s_manage = '
            '%(package:name)s.application.manage:run\n'
            ''
        )

    def gather_recipes(self):
        self.add_recipe(ProjectRecipe(False))

    def gather_tasks(self):
        self.add_task(CreateDataDir)
        self.add_task(FrontendIni)
        self.add_task(Serve)
        self.add_task(BaelfireInitFile)
        self.add_task(InitPy)
        self.add_task(Routes)
        self.add_task(ProjectTemplates)
        self.add_task(Settings)
        self.add_task(Develop)
        self.add_task(Shell)
        self.add_task(UwsgiStart)
        self.add_task(UwsgiStop)
        self.add_task(UwsgiRestart)
        self.add_task(Tests)
        self.add_task(TestsList)
        self.add_task(Coverage)
        self.add_task(AlembicData)
        self.add_task(AlembicMigration)
        self.add_task(AlembicRevision)
        self.add_task(TestFixtures)
        self.add_task(TestCases)
        self.add_task(TestSettings)
        self.add_task(RedmeFile)
        self.add_task(ManagePy)

    def _filter_task(self, task):
        return task.get_path().startswith(self.prefix)


def run():
    Application(recipe=HatakRecipe())()
